# Last edited R.LOISON 2023-07-04

start <- Sys.time()
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

### Libraries needed for the analysis
library(readr)
library(lme4)
library(ggplot2)
library(robustlmm)
library(lmerTest)

### dataset with seed cotton yield
ess1820 <- read_delim("EssaisGE RendementCG.txt", 
                      "\t", escape_double = FALSE, trim_ws = TRUE)
ess1820$AMEX <- as.factor(ess1820$AMEX)
ess1820$Rep  <- as.factor(ess1820$Rep)
ess1820$DateSemis <- as.factor(ess1820$DateSemis)
ess1820$Variete <- as.factor(ess1820$Variete)

# Robust model
model2 <- rlmer(sqrt(Rdt)~ -1+ DateSemis:Variete + (1|AMEX) + (1|AMEX:Rep) + (1|AMEX:Variete) + (1|AMEX:DateSemis) +  (1|AMEX:Rep:DateSemis),data=ess1820)
summary(model2)
str(summary(model2))
sqrt(diag(summary(model2)$vcov))


# Classical model
library(lmerTest)
model3 <- lmerTest::lmer(sqrt(Rdt)~ -1+ DateSemis*Variete + 
                           (1|AMEX) + (1|AMEX:Rep) + (1|AMEX:Variete) + 
                           (1|AMEX:DateSemis) +  (1|AMEX:Rep:DateSemis),data=ess1820)
anova(model3)
summary(model3)
############################################
#### Description des effets al?atoires #####
############################################
# Variance des effets al?atoires
summary(model3)

# Tests de vraissemblance pour tester chacun des effets al?toire pr?sents
# Random effects:
#    Groups             Name        Variance Std.Dev.
# 1. AMEX:Variete       (Intercept)  0.6145  0.7839  
# 2. AMEX:Rep:DateSemis (Intercept)  5.7024  2.3880  
# 3. AMEX:Rep           (Intercept)  0.0000  0.0000  
# 4. AMEX:DateSemis     (Intercept)  9.9996  3.1622  
# 5. AMEX               (Intercept) 19.6377  4.4314  
#   Residual                        15.2145  3.9006  
# Number of obs: 288, groups:  
#    AMEX:Variete, 48; AMEX:Rep:DateSemis, 36; AMEX:Rep, 18; AMEX:DateSemis, 12; AMEX, 6
ranova(model3)

# variance d'une combinaison lin?aire de param?tres
# Var(L beta) = L Var(beta) L' 
# avec L matrice de coefficients 
# o? chaque ligne est un vecteur de coefficients d'une combinaison lin?aire de param?tres 
# beta le vecteur des param?tres
# Var(beta) la matrice de variance covariance des param?tres
# 
beta <- summary(model2)$coefficients[,1]
covbeta <- summary(model2)$vcov
p <- dim(summary(model2)$vcov)[1]

###################################################
#### 1. Analysis per cultivar and planting date  ##
###################################################


### Average performance per planting date
l01 = rep(c(1/8,0),8) ## Early planting
e01 <- as.numeric(l01 %*% beta)
e01

l02 = rep(c(0,1/8),8) ## Delayed planting
e02 <- as.numeric(l02 %*% beta)
e02

### Differences between cultivars for each planting date
l = c(1,0,-1,0,0,rep(0,p-5)) ## contrasts to compute the difference between 2 cultivars' seed cotton yield for each planting date
e3 <- l %*% beta
vare3 <- t(l) %*% covbeta %*% l
sqrt(vare3)
ppas_inter <- as.numeric(qtukey(p=0.95,nmeans=8,df=35,nranges=2) *sqrt(vare3/2)) # Tukey's HSD at 95%

### Data frame for the Figure 1
result <- data.frame(Traitement =names(beta),
                     sqrt_Rdt=as.numeric(beta), 
                     demi_MSD=ppas_inter/2)
result$min <- result$sqrt_Rdt-result$demi_MSD
result$max <- result$sqrt_Rdt+result$demi_MSD
result$Rdt_moyen <- (result$sqrt_Rdt)^2
result$Rdt_min <- (result$min)^2
result$Rdt_max <- (result$max)^2
result$DateSemis <- substr(result$Traitement,10,10)
result$Variete <- substr(result$Traitement,19,nchar(result$Traitement))

# New facet label names for supp variable
DateSemis.labs <- c("Early planting date", "Delayed planting date (+ 2 weeks)")
names(DateSemis.labs) <- c("1", "2")

# Ranking of cultivars for delayed planting
result$order_variete_Late[result$DateSemis=="2"] <- rank(result$sqrt_Rdt[result$DateSemis=="2"])
a <- result[result$DateSemis=="2",c("Variete","order_variete_Late")]

# Ranking of cultivars for early planting
result$order_variete_Early[result$DateSemis=="1"] <- rank(result$sqrt_Rdt[result$DateSemis=="1"])
b <- result[result$DateSemis=="1",c("Variete","order_variete_Early")]

result <- result[,1:(dim(result)[2]-2)] # removal of the 2 columns w/ rankings from data frame "result"
library(plyr)
c <- join(a,b)
result <- join(result,c) # data frame with 2 extra columns for the ranking

# Values for average yield in early and delayed planting conditions
dummy2 <- data.frame(DateSemis = c("1","2"), Z = c(e01^2,e02^2))

# Object for Figure 1
graphic_comparaison_variete_sep_par_date_semis <- ggplot(result, aes(y=reorder(Variete,order_variete_Early), x=Rdt_moyen))+
  geom_errorbarh(aes(color=DateSemis,xmin=Rdt_min,xmax=Rdt_max),height=0,lwd=1.1)+
  geom_vline(data=dummy2,aes(xintercept = Z), linetype=2,col=c("#1FA22E","#E2007A"),lwd=0.8)+
  geom_point(aes(color=DateSemis),size=3)+
  ylab('Cultivars')+ xlab(expression(paste("Seed cotton yield (kg ",ha^{-1},")")))+
  scale_x_continuous(breaks=c(600,round(dummy2$Z[2],0),800,1000,1200,
                              round(dummy2$Z[1],0),1600))+
  scale_color_manual(values=c("#1FA22E", "#E2007A"),name = "Planting date", 
                     labels = c("Early", "Late"))+
  theme_bw()+
  theme(axis.text.x = element_text(color = c(1,"#E2007A",1,1,1,"#1FA22E",1)))+
  theme(panel.grid = element_blank())

graphic_comparaison_variete_sep_par_date_semis # print the graphic

# Create a PDF file from the R object
pdf(file="Figure_1.pdf",width=8,height=4)
graphic_comparaison_variete_sep_par_date_semis
dev.off()


cairo_ps("Figure_1.eps")
graphic_comparaison_variete_sep_par_date_semis
dev.off()


############################################################################################
#### 2. Linear regression of the evolution of the proportion of early vs. delayed planting #
############################################################################################

# Data frame with proportion of early planting from 2000 to 2019
dt_sp <- read.csv("pourcentage_early_planting.csv",header=T,sep=",")
plot(dt_sp)

model_sp2 <- glm(pourc_gpe1et2 ~ year,data= dt_sp,family="quasibinomial")
#plot(model_sp2)
summary(model_sp2)

model_sp2bis <- lm(pourc_gpe1et2 ~ year,data= dt_sp)
library(MASS)

best_lm <- stepAIC(model_sp2bis,trace=T)
summary(best_lm)

model_sp2bis <- lm(pourc_gpe1et2 ~ 1,data= dt_sp)
pente <-summary(model_sp2bis)
str(pente)
cor(fitted(model_sp2bis),dt_sp$pourc_gpe1et2)
predict(model_sp2bis,Year=seq(2020:2050),interval="predict")

aov(model_sp2bis)
fitted(model_sp2bis)

pdf("Figure_2.pdf")
plot(100- 100*dt_sp$pourc_gpe1et2~dt_sp$year,pch=16,xlab="Year",axes=F,ylim=c(0,60),
     ylab="Percentage of late planting (%)")
ticks = c(0,23.9,40,60)
axis(side = 2, at = ticks)
ticks1 = c(2000,2005,2010,2015,2020,2022)
axis(side=1,at=ticks1,labels=ticks1)+
abline(a=as.numeric(23.9),b=0,col=2,lwd=2)
dev.off()

##################################################################
#### 3. Cultivars' seed cotton yields at 76.1% of early planting #
####    estimated with linear estimates methods                  #
##################################################################

# Preliminaires (mis en commentaires)
# # exemple 1 : erreur-type de DateSemis1:VarieteALLEN 51-106
# l = c(1,rep(0,p-1))
# # on retrouve bien l'erreur-type de la moyenne ajust?e de la vari?t? 1
# # sous R les vecteurs sont apparemment des vecteurs-colonne, c'est donc l' Var(beta) l
# e1    <- t(l) %*% beta
# vare1 <- t(l) %*% covbeta %*% l
# sqrt(vare1)

# Erreur-type d'une moyenne pond?r?e 75.5% DateSemis1:VarieteALLEN 51-106 et 25 % DateSemis2:VarieteALLEN 51-106
psp <- 0.761 # proportion de semis precoce
l = c(psp,1-psp,rep(0,p-2))
# e3 <- t(l) %*% beta
# vare3 <- t(l) %*% covbeta %*% l
# sqrt(vare3)
# ces r?sultats sont aussi les m?mes (? la 4e d?cimale) que ceux obtenus avec proc mixed et lsmeans variete, om=poids
# avec l'analyse classique, pas robuste

# exemple 3 bis : comme le dispositif est ?quilibr?, on retrouve les m?mes erreurs-type pour toutes les moyennes par vari?t?
nv<-8
noms <- rep(".",nv)
L <- matrix(0,p/2,p)
for (i in 1:(p/2)){
  print(i)
  L[i,2*i-1] <- l[1]
  L[i,2*i]   <- l[2]
  noms[i]<-levels(ess1820$Variete)[i]
}
print(L)
e3 <- L %*% beta
row.names(e3)<-noms
e3
vare3 <- L %*% covbeta %*% t(L)
sqrt(diag(vare3))


l_global <- rep(c(psp/8,(1-psp)/8),8)
e3bis <- as.numeric(l_global %*% beta)
e3bis ## perf globale ? 75.5% de semis pr?coce

e3 # perf par vari?t? ? 75.5% de semis pr?coce



# exemple 4 :
# on peut aussi calculer les erreurs-types des diff?rences
ld <- L[2,]-L[1,]
e4 <- ld %*% beta
vare4 <- t(ld) %*% covbeta %*% ld
sqrt(vare4)

# exemple 4 bis :
# on fait cela sur tous les couples de vari?t?s
# nombre de vari?t?s
nv <- p/2
Ld <- matrix(0,nv*(nv-1)/2,p)
# liste des noms ? donner aux contrastes ?l?mentaires que l'on va estimer
noms <- rep(".",nv*(nv-1)/2)
k <- 0
for (i in 2:(p/2)){
  for (j in 1:(i-1)){
    k <- k+1
    #   print(c(i,j,k))
    Ld[k,] <- L[i,]-L[j,]
    noms[k]<-paste(levels(ess1820$Variete)[i],levels(ess1820$Variete)[j],sep=" :-: ")
  }
}
e4 <- Ld %*% beta
row.names(e4)<-noms
vare4 <- Ld %*% covbeta %*% t(Ld)
row.names(vare4)<-noms
colnames(vare4)<-noms
sqrt(diag(vare4))

# diff?rences standardis?es et multipli?es par racine de 2, ? comparer au qtukey
diffstd2 <- e4/sqrt(diag(vare4)/2)
qtukey(p=0.95,nmeans=8,df=35,nranges=1)

# p-values 
ptukey(q=diffstd2,nmeans=8,df=35,nranges=1,lower.tail = FALSE)

# plus petite amplitude significative pour les graphiques
# au lieu de la multiplier par l'erreur-type d'une moyenne
# qui n'est pas int?ressante dans le cas d'un mod?le mixte
# on la multiplie par l'erreur-type de la diff?rence entre moyennes, 
# divis?e par racine de 2
ppas <- qtukey(p=0.95,nmeans=8,df=35,nranges=1) * sqrt(diag(vare4)/2)


# liste des diff?rences significatives at 5%
(e4/sqrt(diag(vare4)/2))>qtukey(p=0.95,nmeans=8,df=35,nranges=1)


### Valeur min et max :
min <- (e3-0.5*ppas[1])^2
max <- (e3+0.5*ppas[1])^2
moy <- e3^2
result2 <- data.frame(min, max, moy)
result2$Variete <- row.names(e3)
result2$order_variete <-rank(result2$moy)

#library(ggplot2)

graphic_comparaison_variete_75p <- ggplot(result2, aes(y=reorder(Variete,order_variete), x=moy))+
  geom_vline(aes(xintercept = e3bis^2), linetype=2,col=2,lwd=0.8)+
  geom_errorbarh(aes(xmin=min,xmax=max),height=0,lwd=1.1)+
  geom_point(size=3)+
  ylab('Cultivars')+ xlab(expression(paste("Seed cotton yield (kg ",ha^{-1},")")))+
  
  scale_x_continuous(breaks=c(1000,round(e3bis^2,0),1400))+
  theme_bw()+
  theme(axis.text.x = element_text(color = c(1,2,1)))+
  theme(panel.grid = element_blank())

graphic_comparaison_variete_75p

pdf(file="Figure_3.pdf",width=8,height=4)
graphic_comparaison_variete_75p
dev.off()  


##############################################
#### 4. Same as section 3 for any proportion ####
##############################################

# On fait la combinaison lin?aire pour 0% de semis pr?coce (valeurs ? l'origine pour les vari?t?s)
# puis celle ? 100% de semis precoce
# on trace des segment entre ordonnee et ? l'origine et ? 100% par vari?t?
# pour les MSD on fait le calcul pour chaque valeur de proportion de semis pr?coce



# la m?me chose est faisable avec d'autres coefficients que 3/4 1/4
# 1 0 pour les semis precoces
# 0 1 pour les semis tardifs
# ou encore une boucle pour obtenir la serie de ppas dont on a besoin pour le graphique 
# o? le pourcentage de semis pr?coces varie en continu

# Performance des varietes a 0 et 100pc de semis precoce
psp <- 0.0 # proportion de semis pr?coce
l = c(psp,1-psp,rep(0,p-2))
psp2 <- 1 
l2 =c(psp2,1-psp2,rep(0,p-2))
# example 3 bis : balanced dataset, on retrouve les m?mes erreurs-type pour toutes les moyennes par variete
noms <- rep(".",nv)
L <- matrix(0,p/2,p)
for (i in 1:(p/2)){
  print(i)
  L[i,2*i-1] <- l[1]
  L[i,2*i]   <- l[2]
  noms[i]<-levels(ess1820$Variete)[i]
}

L2 <- matrix(0,p/2,p)
for (i in 1:(p/2)){
  print(i)
  L2[i,2*i-1] <- l2[1]
  L2[i,2*i]   <- l2[2]
  noms[i]<-levels(ess1820$Variete)[i]
}
#print(L)
e0pc <- L %*% beta
e100pc <- L2 %*% beta
row.names(e0pc)<-noms
row.names(e100pc)<-noms

library(plyr)
dt0 <- data.frame(e0pc)
dt0$Variete <- row.names(dt0)
dt100 <- data.frame(e100pc)
dt100$Variete <- row.names(dt100)
dt_final <- join(dt0,dt100)
dt_final$intercept <- dt_final$e0pc
dt_final$slope <- dt_final$e100pc-dt_final$e0pc



# MSD computed pour 0 and 1 early planting
prop <- seq(0,1,0.01)
output <- data.frame(proportion=rep(NA,length(prop)),MSD=rep(NA,length(prop)))
for(i in 1:length(prop)){
  psp <- prop[i]
  l = c(-psp,psp-1,psp,1-psp,rep(0,(p-4)))
  
  vare4 <- t(l) %*% covbeta %*% l
  output$MSD[i] <- qtukey(p=0.95,nmeans=8,df=35,nranges=1) * sqrt(diag(vare4)/2)
  output$proportion[i]<- psp
}

plot(data=output,MSD~proportion)

test<-expand.grid(dt_final$Variete,output$proportion)
names(test)<- c("Variete","proportion")
test2 <-join(test,output)

dt_finalllll <- join(test2,dt_final)


# Figure 4 ####
dt_final2 <- dt_final[,c("Variete","e0pc","e100pc")]
dt_final2$e0pc <- dt_final2$e0pc^2
dt_final2$e100pc <- dt_final2$e100pc^2

dt_final3 <- expand.grid("Variete"=dt_final2$Variete,"proportion"=prop)

dt_final4 <- join(dt_final3,dt_final2)
dt_final4$RDT <- sqrt(dt_final4$proportion*dt_final4$e100pc+(1-dt_final4$proportion)*dt_final4$e0pc)
dt_final5 <- dt_final4[order(dt_final4$Variete,dt_final4$proportion),]
dt_final6 <- dt_final5[,c(1,2,5)]
dt_finalllll <- join(dt_finalllll,dt_final6)

graphic_all_prop <- ggplot(data=output, aes(x=100*(1-proportion),col=Variete)) +
  #   geom_vline(xintercept = 0.755,linetype="dashed",col=2)+
  
  labs(x="Percentage of late planting (%)", y= expression(paste(sqrt("Seed cotton yield")," (",kg^{0.5}," ",ha^{-0.5},")")))+
  geom_linerange(data=dt_finalllll , aes(ymin =42, ymax =42+MSD),lwd=0.5,col="grey50")+
  geom_linerange(aes(x=23.9,ymin =21, ymax =41),lwd=0.5,col="grey50",linetype="dashed")+
  #geom_segment(aes(x = 0.755, xend = 0.755, y = 21, yend = 40.8),size=0.5,linetype="dashed")+
  #geom_linerange(aes(x=24.5,ymin=31.5, ymax=31.5+3.306044),lwd=1.5,col="grey50")+
  geom_line(data=dt_finalllll,aes(x = 100*(1-proportion), y=RDT,color=Variete),size=0.7)+
  #geom_abline(data=dt_finalllll,aes(x=proportion,slope=slope,intercept=intercept,color=Variete),size=1)+
  scale_x_continuous(breaks=c(0,23.9,40,60,80,100),limits=c(0,100))+
  scale_y_continuous(breaks=c(25,30,35,40,42,44,46),
                     labels=c(25,30,35,40,0,2,4),limits=c(21,47))+
  guides(col=guide_legend("Cultivars",keyheight=3,
                          default.unit="mm"))+
  annotate("text",x= 40, y= 47, size=3,
           label="Minimum Significant Difference \n(Tukey 95% significance)",
           col="grey50")+
  theme_bw()+
  theme(axis.text.x = element_text(color = c(1,"grey50",1,1,1,1)),
        axis.text.y=element_text(color=c(rep(1,4),rep("grey50",5))),
        legend.text = element_text(size = 8, colour = 1))+
  theme(panel.grid= element_blank())
graphic_all_prop

pdf("Figure_4.pdf",width=7,height=5)
graphic_all_prop
dev.off()

(algo_timing <- Sys.time()-start)
